#include <iostream>
#include <cmath>

// This simple example solves the following constrainted optimization problem:
//
// argmin_{x} \sum_{i = 0}^{4} x_{i}
// s. t. 61 / x_{0}^3
//     + 37 / x_{1}^3
//     + 19 / x_{2}^3
//     + 7 / x_{3}^3
//     + 1 / x_{4}^3
//     - 1 < 0

// The API of the optimizer is entirely defined in the optlab header
#include <optlab.h>

int main(int argc, char **argv)
{
  // Maximum number of iterations
  int maxIter = 10;

  // number of variables and constraints
  int numVariables = 5;

  // initial value of the variables in the design space
  std::vector<double> initialPoint(numVariables, 5.0);

  // lower bound for design variables (all set to 1)
  std::vector<double> lowerBound(numVariables, 1.0);

  // upper bound for design variables (all set to 10)
  std::vector<double> upperBound(numVariables, 10.0);

  // Initialize the MMA optimizer
  optlab::mma::initialize(initialPoint, lowerBound, upperBound);

  // Set some options for MMA
  //optlab::mma::option::setNumber("General.Verbosity", 4);
  //optlab::mma::option::setNumber("General.SaveHistory", 0);
  //optlab::mma::option::setNumber("SubProblem.useHessian", 1);

  // Get iteration count (here it will be 1 - could be different in case of restart)
  int it;
  optlab::mma::getOuterIteration(it);

  while(it <= maxIter){

    // get (copy of) current point
    std::vector<double> x;
    optlab::mma::getCurrentPoint(x);

    // compute objective function and constraints 
    // at point `x' in the design space
    double objective=0.0;
    std::vector<double> constraints(1);
    for(unsigned int i = 0; i < numVariables; i++)
      objective += x[i];
    constraints[0] = 61.0 / std::pow(x[0], 3)
      + 37.0 / std::pow(x[1], 3)
      + 19.0 / std::pow(x[2], 3)
      + 7.0 / std::pow(x[3], 3)
      + 1.0 / std::pow(x[4], 3)
      - 1.0;

    // as well as their sensitivity with respect to design variables at `x'
    std::vector<double> grad_objective(numVariables);
    std::vector<double> grad_constraints(constraints.size() * numVariables);
    for(unsigned int i = 0; i < grad_objective.size(); i++)
      grad_objective[i] = 1.0;
    grad_constraints[0] = -3.*61. / std::pow(x[0], 4);
    grad_constraints[1] = -3.*37. / std::pow(x[1], 4);
    grad_constraints[2] = -3.*19. / std::pow(x[2], 4);
    grad_constraints[3] = -3.*7. / std::pow(x[3], 4);
    grad_constraints[4] = -3.*1. / std::pow(x[4], 4);

    // call MMA and update the current point
    optlab::mma::updateCurrentPoint(constraints, grad_objective, grad_constraints);

    std::cout << "iteration: " << it << "  "
              << "objective: " << objective << std::endl;

    optlab::mma::getOuterIteration(it);
  }

  // This should be called at the end
  optlab::mma::finalize();
}
